import Vue from "vue";
import Vuex from "vuex";
import createPersistedState from "vuex-persistedstate"; // 持久化存储插件

import exam from "./modules/exam";
import answer from "./modules/answer";

Vue.use(Vuex);

export default new Vuex.Store({
  plugins: [createPersistedState({
    storage: window.sessionStorage,
    reducer: state => {
      return {
        user: { }
      };
    }
  })],
  state: {
    current: Date.now(), // 当前时间
    intervalID: 0,
    screenLoading: false // 显示全屏加载中
  },
  getters: {
    hidden() {
      if (typeof document.hasFocus !== "function") {
        return document.hidden;
      }
      return !document.hasFocus();
    }
  },
  mutations: {
    startComputeCurrent(state) {
      state.intervalID = setInterval(() => {
        state.current = Date.now();
      }, 500);
    },
    stopComputeCurrent(state) {
      clearInterval(state.intervalID);
      state.intervalID = 0;
    },
    toggleLoading(state, value) {
      state.screenLoading = value;
    },
    updateItem(state, { field, value }) {
      state[field] = value;
    }
  },
  modules: {
    exam,
    answer
  }
});
